#include <stdio.h>
#include <stdlib.h>
#include <termios.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <X11/Xcms.h>
#include <X11/Xutil.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <SDL.h>

#include "vt100.h"
#include "async.h"
#include "main.h"

#define WIDTH	80
#define HEIGHT  43	

#define BUFFER_LEN	1024
#define SPEED		8
#define MAX_TIMER	64	

#define BOX_W		200
#define BOX_H		100

#define SHELL		"/bin/bash"

SDL_Surface *d_screen;
SDL_Surface *d_back;
unsigned char font[];

term_t main_term;

int cursor = 0;

int master, slave;
pid_t child_pid;
int d_dirt = 1;

typedef struct {
 int r,g,b;
} color_t;


float dest_x, dest_y, dest_w, dest_h;
int floating_x, floating_y;
int floating = 0;
int timer, timer_switch = 0;
int mousemotion = 0;

color_t color_values[] = {
 {0x00, 0x00, 0x00},
 {0xaf, 0x00, 0x00},
 {0x00, 0xaf, 0x00},
 {0xaf, 0xaf, 0x00},

 {0x1f, 0x1f, 0xaf},
 {0xaf, 0x00, 0xaf},
 {0x00, 0xaf, 0xaf},
 {0xaf, 0xaf, 0xaf},

 {0x00, 0x00, 0x00},
 {0xff, 0x00, 0x00},
 {0x00, 0xff, 0x00},
 {0xff, 0xff, 0x00},

 {0x1f, 0x1f, 0xff},
 {0xff, 0x00, 0xff},
 {0x00, 0xff, 0xff},
 {0xff, 0xff, 0xff}
};

Uint32 colors[16];

void init_colors(void) {
 int i;
 for(i=0;i<16;i++) {
  colors[i] = SDL_MapRGB(d_back->format, color_values[i].r, 
                                           color_values[i].g,
			                   color_values[i].b);
 }
}

void blit_scaled(int src_x, int src_y, int src_w, int src_h, SDL_Surface *src){
 float qx, qy;
 int x, y;
 int r,g,b;
 int px, py;

 union {
  Uint8  *b;
  Uint32 *l;
 } pix, pix2, apix, apix2;


 if(src_x < 0) src_x = 0;
 if(src_y < 0) src_y = 0;

 if((src_x+src_w) > d_screen->w)
  src_x = d_screen->w - src_w;

 if((src_y+src_h) > d_screen->h)
  src_y = d_screen->h - src_h;
 
 qx = (float)src_w / (float)d_screen->w;
 qy = (float)src_h / (float)d_screen->h;

 for(y=0;y<d_screen->h;y++) {
  py = (int)((float)src_y + (qy * (float)y));
  pix.b = (Uint8 *)src->pixels+(py*src->pitch);
  pix2.b = (Uint8 *)d_screen->pixels+(y*d_screen->pitch);
  for(x=0;x<d_screen->w;x++) {
   px = (int)((float)src_x + (qx * (float)x));

   apix.b = (Uint8 *)pix.b + (px*src->format->BytesPerPixel);  
   apix2.b= (Uint8 *)pix2.b+ (x*d_screen->format->BytesPerPixel);
   
   *apix2.l = *apix.l;

  }
 }
}

void d_char(int x, int y, char_t *c) {
 int fg, bg;
 Uint32 color, fg_color, bg_color;
 int real_x, real_y;
 int t,s;
 Uint32 mask;
 union {
  Uint8  *b;
  Uint32 *l;
 } pix, pix2;

 bg = c->col&0xf;
 fg= (c->col>>4)&0xf;

 if(bg ==0)
  bg_color = SDL_MapRGB(d_back->format, 0xff, 0xff, 0xff);
 else
  bg_color = colors[bg];

 if(fg == 7)
  fg_color = SDL_MapRGB(d_back->format, 0x00, 0x00, 0x00);
 else
  fg_color = colors[fg];

 real_x = x * 8;
 real_y = y * 8;

 pix.b = (Uint8 *)(d_back->pixels+
                   ((real_x)*d_back->format->BytesPerPixel) +
		   ((real_y)*d_back->pitch));

 mask = (d_back->format->Rmask | 
         d_back->format->Gmask |
 	 d_back->format->Bmask |
	 d_back->format->Amask)^~0;

 for(t=0;t<8;t++)
  for(s=0;s<8;s++) {
   if(((font[ ((c->text-0x20) * 8)+t]>>(8-s)) &1) == 1) 
    color = fg_color; 
   else 
    color = bg_color;
   if(c->text == 0) color = bg_color;
   pix2.b = &pix.b[(t * d_back->pitch)+(s*d_back->format->BytesPerPixel)];
   *pix2.l&=mask;
   *pix2.l|=color;
  }


}

void d_drawcursor(void) {
 int real_x, real_y;
 int t,s;
 Uint32 mask;
 Uint32 color;
 union {
  Uint8  *b;
  Uint32 *l;
 } pix, pix2;

 if(cursor == 1) { 


  real_x = main_term.cursor_x * 8;
  real_y = main_term.cursor_y * 8;

  pix.b = (Uint8 *)(d_back->pixels+
                   ((real_x)*d_back->format->BytesPerPixel) +
		   ((real_y)*d_back->pitch));

  mask = (d_back->format->Rmask | 
          d_back->format->Gmask |
       	  d_back->format->Bmask |
	  d_back->format->Amask)^~0;

  color = SDL_MapRGB(d_back->format, 0x00, 0x00, 0x00);

  for(t=0;t<8;t++)
   for(s=0;s<8;s++) {
    pix2.b = &pix.b[(t * d_back->pitch)+(s*d_back->format->BytesPerPixel)];
    *pix2.l&=mask;
    *pix2.l|=color;
   }
 } else
  d_char(main_term.cursor_x, main_term.cursor_y, 
         &main_term.chars[main_term.cursor_x+(WIDTH*main_term.cursor_y)]);
}
void d_drawscreen(int xx, int yy, int ww, int hh) {
 int x, y;
 if(main_term.dirty == 1) {
  for(y=0;y<HEIGHT;y++)
   for(x=0;x<WIDTH;x++) 
    d_char(x,y, &main_term.chars[x+(WIDTH*y)]);
   
  main_term.dirty = 0;
 }
 d_drawcursor();
 blit_scaled(xx,yy,ww,hh, d_back);
 SDL_UpdateRect(d_screen, 0,0,0,0);
}

void q_input(void) {
 SDL_Event event;
 struct winsize wsize;

 while(SDL_PollEvent(&event)) { 
  switch(event.type) {
   case SDL_KEYDOWN:
    if((SDL_GetModState() & KMOD_LALT)==KMOD_LALT) {
     if(floating == 0) {
      floating_x = main_term.cursor_x;
      floating_y = main_term.cursor_y;
      floating = 1;
     }
     switch(event.key.keysym.sym) {
     /* {{{ */
      case SDLK_UP:
       timer_switch = 1;
       timer = 0;
       dest_w = BOX_W;
       dest_h = BOX_H;
       floating_y--;
       if(floating_y<0)
	floating_y = 0;
       dest_x = (floating_x *8) - (BOX_W/2);
       dest_y = (floating_y *8) - (BOX_H/2);
       mousemotion = 1;
       break;
      case SDLK_DOWN:
       timer_switch = 1;
       timer = 0;
       dest_w = BOX_W;
       dest_h = BOX_H;
       floating_y++;
       if(floating_y>=HEIGHT)
	floating_y = HEIGHT-1;
       dest_x = (floating_x *8) - (BOX_W/2);
       dest_y = (floating_y *8) - (BOX_H/2);
       mousemotion = 1;
       break;
      case SDLK_LEFT:
       timer_switch = 1;
       timer = 0;
       dest_w = BOX_W;
       dest_h = BOX_H;
       floating_x--;
       if(floating_y<0)
	floating_x = 0;
       dest_x = (floating_x *8) - (BOX_W/2);
       dest_y = (floating_y *8) - (BOX_H/2);
       mousemotion = 1;
       break;
      case SDLK_RIGHT:
       timer_switch = 1;
       timer = 0;
       dest_w = BOX_W;
       dest_h = BOX_H;
       floating_x++;
       if(floating_x>=WIDTH)
	floating_x = WIDTH-1;
       dest_x = (floating_x *8) - (BOX_W/2);
       dest_y = (floating_y *8) - (BOX_H/2);
       mousemotion = 1;
       break;
       /* }}} */
     }
    } else {
     floating = 0;
    switch(event.key.keysym.sym) {
     /* {{{ */
     case SDLK_ESCAPE:
      main_term.char_out('\33', (struct term_t *)&main_term);
      break;
     case SDLK_RETURN:
      main_term.char_out('\r', (struct term_t *)&main_term);
      break;
     case SDLK_LEFT:
      vt_send((struct term_t *)&main_term, K_LT);
      break;
     case SDLK_RIGHT:
       vt_send((struct term_t *)&main_term, K_RT);
       break;
     case SDLK_UP:
      vt_send((struct term_t *)&main_term, K_UP);
      break;
     case SDLK_DOWN: 
      vt_send((struct term_t *)&main_term, K_DN);
      break;
     case SDLK_PAGEUP:
      vt_send((struct term_t *)&main_term, K_PGUP);
      break;
     case SDLK_PAGEDOWN:
      vt_send((struct term_t *)&main_term, K_PGDN);
      break;
     case SDLK_HOME:
      vt_send((struct term_t *)&main_term, K_HOME);
      break;
     case SDLK_INSERT:
      vt_send((struct term_t *)&main_term, K_INS);
      break;
     case SDLK_DELETE:
      vt_send((struct term_t *)&main_term, K_DEL);
      break;
     case SDLK_END:
      vt_send((struct term_t *)&main_term, K_END);
      break;
     case SDLK_TAB:
      main_term.char_out(main_term.control_keys[CONTROL_TAB],
       (struct term_t *)&main_term);
      break;
     case SDLK_BACKSPACE:
      main_term.char_out(main_term.control_keys[CONTROL_BACKSPACE],
       (struct term_t *)&main_term);
      break;
     case SDLK_NUMLOCK:
     case SDLK_CAPSLOCK:
     case SDLK_SCROLLOCK:
     case SDLK_RSHIFT:
     case SDLK_LSHIFT:
     case SDLK_RCTRL:
     case SDLK_LCTRL:
     case SDLK_RALT:
     case SDLK_LALT:
     case SDLK_RMETA:
     case SDLK_LMETA:
     case SDLK_LSUPER:
     case SDLK_RSUPER:
     case SDLK_MODE:
     case SDLK_COMPOSE:
      break;
     default:
      main_term.char_out(event.key.keysym.unicode, (struct term_t *)&main_term);
     break; 
     /* }}} */
    }

  }
 }
 }
}

void new_shell(void) {
 struct winsize wsize;
 char slave_name[16];

 if(openpty(&master, &slave, slave_name, NULL, &main_term.ws_conf)<0) {
  perror("openpty");
  exit(1);
 }

 if((child_pid = fork())==0) {
  close(master);
  login_tty(slave);

  setenv("TERM", "Eterm", 1); // XXX doesn't include page-up/down


/*  setenv("TERMCAP","vt100term|any ansi terminal with pessimistic assumptions:\\"
	           ":co#80:li#24:cl=50\\E[;H\\E[2J:bs:am:cm=\\E[%i%d;%dH:\\"
	           ":nd=\\E[C:up=\\E[A:ce=\\E[K:ho=\\E[H:pt:", 1); */


  execl(SHELL, SHELL, (char *)0);
 }
 fcntl(master, F_SETFL, fcntl(master, F_GETFL,0)|O_NONBLOCK); 
 wsize.ws_col = WIDTH;
 wsize.ws_row = HEIGHT;
 ioctl(slave, TIOCSWINSZ, &wsize);
}

void io(void) {
 unsigned char buffer[BUFFER_LEN];
 int len;
 int i;

 while((len = read(master, buffer, BUFFER_LEN))>0) {
  for(i=0;i<len;i++)
   vt_out((struct term_t *)&main_term, buffer[i]);
 }

}

int main(void) {
 int lcursor_x, lcursor_y;
 float fx, fy, fw, fh;

 SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
 if((d_screen = SDL_SetVideoMode(WIDTH*8, HEIGHT * 8, 24, 0))<=0) {
  fprintf(stderr, "%s\n", SDL_GetError());
  exit(0);
 }

 d_back = SDL_CreateRGBSurface(0, WIDTH*8,HEIGHT*8,24,
  d_screen->format->Rmask,
  d_screen->format->Gmask,
  d_screen->format->Bmask,
  d_screen->format->Amask);

 init_colors();

 SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY, 
                     SDL_DEFAULT_REPEAT_INTERVAL);
 SDL_EnableUNICODE(1);

 init_term((struct term_t *)&main_term, callback, WIDTH, HEIGHT);
 main_term.control_keys[CONTROL_BACKSPACE] = BACKSPACE;
 main_term.control_keys[CONTROL_QUIT] = QUIT;
 main_term.control_keys[CONTROL_DOWN] = 0xa;
 main_term.control_keys[CONTROL_TAB] = TAB;
 main_term.control_keys[CONTROL_AAA] = 1;

 new_shell();

 SDL_SetTimer(400, sig_alarm);
 signal(SIGCHLD, sig_child);
 lcursor_x = main_term.cursor_x;
 lcursor_y = main_term.cursor_y;
 fx = 0;
 fy = 0;
 fw = 10;
 fh = 10;
 dest_x = 0;
 dest_y = 0;
 dest_w = d_screen->w;
 dest_h = d_screen->h;
 timer = 0;
 timer_switch = 0;
 for(;;) {
  fx+=(dest_x-fx)/4;
  fy+=(dest_y-fy)/4;
  fw+=(dest_w-fw)/4;
  fh+=(dest_h-fh)/4;
  d_drawscreen((int)fx,(int)fy,(int)fw,(int)fh);
  q_input();
  io();
  if((main_term.cursor_x != lcursor_x) |
     (main_term.cursor_y != lcursor_y)) {
   timer_switch = 1;
   timer = 0;
   mousemotion = 0;
  }

  lcursor_x = main_term.cursor_x;
  lcursor_y = main_term.cursor_y;

  if(timer_switch == 1) {
   timer++;
   if(timer == MAX_TIMER)
    timer_switch = 0;
  }
  if(timer < (MAX_TIMER/2)) {
   if(mousemotion == 0) {
    dest_w = BOX_W; 
    dest_h = BOX_H;
    dest_x = (main_term.cursor_x *SPEED) - (BOX_W/2);
    dest_y = (main_term.cursor_y *SPEED) - (BOX_H/2);
   }
  } else {
   dest_x = 0;
   dest_y = 0;
   dest_w = d_screen->w;
   dest_h = d_screen->h;
   mousemotion = 0;
  }
  SDL_Delay(10);
 }
 return 0;
}


unsigned char font[] = { 
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x18, 0x3C, 0x3C, 0x18, 0x18, 0x00, 0x18, 0x00,
 0x6C, 0x6C, 0x6C, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x6C, 0x6C, 0xFE, 0x6C, 0xFE, 0x6C, 0x6C, 0x00,
 0x18, 0x7E, 0xC0, 0x7C, 0x06, 0xFC, 0x18, 0x00,
 0x00, 0xC6, 0xCC, 0x18, 0x30, 0x66, 0xC6, 0x00,
 0x38, 0x6C, 0x38, 0x76, 0xDC, 0xCC, 0x76, 0x00,
 0x30, 0x30, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x18, 0x30, 0x60, 0x60, 0x60, 0x30, 0x18, 0x00,
 0x60, 0x30, 0x18, 0x18, 0x18, 0x30, 0x60, 0x00,
 0x00, 0x66, 0x3C, 0xFF, 0x3C, 0x66, 0x00, 0x00,
 0x00, 0x18, 0x18, 0x7E, 0x18, 0x18, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x30,
 0x00, 0x00, 0x00, 0x7E, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x18, 0x18, 0x00,
 0x06, 0x0C, 0x18, 0x30, 0x60, 0xC0, 0x80, 0x00,
 0x7C, 0xCE, 0xDE, 0xF6, 0xE6, 0xC6, 0x7C, 0x00,
 0x30, 0x70, 0x30, 0x30, 0x30, 0x30, 0xFC, 0x00,
 0x78, 0xCC, 0x0C, 0x38, 0x60, 0xCC, 0xFC, 0x00,
 0x78, 0xCC, 0x0C, 0x38, 0x0C, 0xCC, 0x78, 0x00,
 0x1C, 0x3C, 0x6C, 0xCC, 0xFE, 0x0C, 0x1E, 0x00,
 0xFC, 0xC0, 0xF8, 0x0C, 0x0C, 0xCC, 0x78, 0x00,
 0x38, 0x60, 0xC0, 0xF8, 0xCC, 0xCC, 0x78, 0x00,
 0xFC, 0xCC, 0x0C, 0x18, 0x30, 0x30, 0x30, 0x00,
 0x78, 0xCC, 0xCC, 0x78, 0xCC, 0xCC, 0x78, 0x00,
 0x78, 0xCC, 0xCC, 0x7C, 0x0C, 0x18, 0x70, 0x00,
 0x00, 0x18, 0x18, 0x00, 0x00, 0x18, 0x18, 0x00,
 0x00, 0x18, 0x18, 0x00, 0x00, 0x18, 0x18, 0x30,
 0x18, 0x30, 0x60, 0xC0, 0x60, 0x30, 0x18, 0x00,
 0x00, 0x00, 0x7E, 0x00, 0x7E, 0x00, 0x00, 0x00,
 0x60, 0x30, 0x18, 0x0C, 0x18, 0x30, 0x60, 0x00,
 0x3C, 0x66, 0x0C, 0x18, 0x18, 0x00, 0x18, 0x00,
 0x7C, 0xC6, 0xDE, 0xDE, 0xDC, 0xC0, 0x7C, 0x00,
 0x30, 0x78, 0xCC, 0xCC, 0xFC, 0xCC, 0xCC, 0x00,
 0xFC, 0x66, 0x66, 0x7C, 0x66, 0x66, 0xFC, 0x00,
 0x3C, 0x66, 0xC0, 0xC0, 0xC0, 0x66, 0x3C, 0x00,
 0xF8, 0x6C, 0x66, 0x66, 0x66, 0x6C, 0xF8, 0x00,
 0xFE, 0x62, 0x68, 0x78, 0x68, 0x62, 0xFE, 0x00,
 0xFE, 0x62, 0x68, 0x78, 0x68, 0x60, 0xF0, 0x00,
 0x3C, 0x66, 0xC0, 0xC0, 0xCE, 0x66, 0x3A, 0x00,
 0xCC, 0xCC, 0xCC, 0xFC, 0xCC, 0xCC, 0xCC, 0x00,
 0x78, 0x30, 0x30, 0x30, 0x30, 0x30, 0x78, 0x00,
 0x1E, 0x0C, 0x0C, 0x0C, 0xCC, 0xCC, 0x78, 0x00,
 0xE6, 0x66, 0x6C, 0x78, 0x6C, 0x66, 0xE6, 0x00,
 0xF0, 0x60, 0x60, 0x60, 0x62, 0x66, 0xFE, 0x00,
 0xC6, 0xEE, 0xFE, 0xFE, 0xD6, 0xC6, 0xC6, 0x00,
 0xC6, 0xE6, 0xF6, 0xDE, 0xCE, 0xC6, 0xC6, 0x00,
 0x38, 0x6C, 0xC6, 0xC6, 0xC6, 0x6C, 0x38, 0x00,
 0xFC, 0x66, 0x66, 0x7C, 0x60, 0x60, 0xF0, 0x00,
 0x7C, 0xC6, 0xC6, 0xC6, 0xD6, 0x7C, 0x0E, 0x00,
 0xFC, 0x66, 0x66, 0x7C, 0x6C, 0x66, 0xE6, 0x00,
 0x7C, 0xC6, 0xE0, 0x78, 0x0E, 0xC6, 0x7C, 0x00,
 0xFC, 0xB4, 0x30, 0x30, 0x30, 0x30, 0x78, 0x00,
 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0xFC, 0x00,
 0xCC, 0xCC, 0xCC, 0xCC, 0xCC, 0x78, 0x30, 0x00,
 0xC6, 0xC6, 0xC6, 0xC6, 0xD6, 0xFE, 0x6C, 0x00,
 0xC6, 0xC6, 0x6C, 0x38, 0x6C, 0xC6, 0xC6, 0x00,
 0xCC, 0xCC, 0xCC, 0x78, 0x30, 0x30, 0x78, 0x00,
 0xFE, 0xC6, 0x8C, 0x18, 0x32, 0x66, 0xFE, 0x00,
 0x78, 0x60, 0x60, 0x60, 0x60, 0x60, 0x78, 0x00,
 0xC0, 0x60, 0x30, 0x18, 0x0C, 0x06, 0x02, 0x00,
 0x78, 0x18, 0x18, 0x18, 0x18, 0x18, 0x78, 0x00,
 0x10, 0x38, 0x6C, 0xC6, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xFF,
 0x30, 0x30, 0x18, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x78, 0x0C, 0x7C, 0xCC, 0x76, 0x00,
 0xE0, 0x60, 0x60, 0x7C, 0x66, 0x66, 0xDC, 0x00,
 0x00, 0x00, 0x78, 0xCC, 0xC0, 0xCC, 0x78, 0x00,
 0x1C, 0x0C, 0x0C, 0x7C, 0xCC, 0xCC, 0x76, 0x00,
 0x00, 0x00, 0x78, 0xCC, 0xFC, 0xC0, 0x78, 0x00,
 0x38, 0x6C, 0x64, 0xF0, 0x60, 0x60, 0xF0, 0x00,
 0x00, 0x00, 0x76, 0xCC, 0xCC, 0x7C, 0x0C, 0xF8,
 0xE0, 0x60, 0x6C, 0x76, 0x66, 0x66, 0xE6, 0x00,
 0x30, 0x00, 0x70, 0x30, 0x30, 0x30, 0x78, 0x00,
 0x0C, 0x00, 0x1C, 0x0C, 0x0C, 0xCC, 0xCC, 0x78,
 0xE0, 0x60, 0x66, 0x6C, 0x78, 0x6C, 0xE6, 0x00,
 0x70, 0x30, 0x30, 0x30, 0x30, 0x30, 0x78, 0x00,
 0x00, 0x00, 0xCC, 0xFE, 0xFE, 0xD6, 0xD6, 0x00,
 0x00, 0x00, 0xB8, 0xCC, 0xCC, 0xCC, 0xCC, 0x00,
 0x00, 0x00, 0x78, 0xCC, 0xCC, 0xCC, 0x78, 0x00,
 0x00, 0x00, 0xDC, 0x66, 0x66, 0x7C, 0x60, 0xF0,
 0x00, 0x00, 0x76, 0xCC, 0xCC, 0x7C, 0x0C, 0x1E,
 0x00, 0x00, 0xDC, 0x76, 0x62, 0x60, 0xF0, 0x00,
 0x00, 0x00, 0x7C, 0xC0, 0x70, 0x1C, 0xF8, 0x00,
 0x10, 0x30, 0xFC, 0x30, 0x30, 0x34, 0x18, 0x00,
 0x00, 0x00, 0xCC, 0xCC, 0xCC, 0xCC, 0x76, 0x00,
 0x00, 0x00, 0xCC, 0xCC, 0xCC, 0x78, 0x30, 0x00,
 0x00, 0x00, 0xC6, 0xC6, 0xD6, 0xFE, 0x6C, 0x00,
 0x00, 0x00, 0xC6, 0x6C, 0x38, 0x6C, 0xC6, 0x00,
 0x00, 0x00, 0xCC, 0xCC, 0xCC, 0x7C, 0x0C, 0xF8,
 0x00, 0x00, 0xFC, 0x98, 0x30, 0x64, 0xFC, 0x00,
 0x1C, 0x30, 0x30, 0xE0, 0x30, 0x30, 0x1C, 0x00,
 0x18, 0x18, 0x18, 0x00, 0x18, 0x18, 0x18, 0x00,
 0xE0, 0x30, 0x30, 0x1C, 0x30, 0x30, 0xE0, 0x00,
 0x76, 0xDC, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x10, 0x38, 0x6C, 0xC6, 0xC6, 0xFE, 0x00
};
